"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findPrebuildifyModule = exports.determineNativePrebuildExtension = exports.determineNativePrebuildArch = exports.getNodeArch = exports.uname = exports.isPrebuildifyNativeModule = void 0;
const child_process_1 = require("child_process");
const debug_1 = __importDefault(require("debug"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const node_api_1 = require("../node-api");
const d = debug_1.default('electron-rebuild');
function isPrebuildifyNativeModule(devDependencies) {
    // eslint-disable-next-line no-prototype-builtins
    return devDependencies.hasOwnProperty('prebuildify');
}
exports.isPrebuildifyNativeModule = isPrebuildifyNativeModule;
/**
 * Runs the `uname` command and returns the trimmed output.
 *
 * Copied from `@electron/get`.
 */
function uname() {
    return child_process_1.execSync('uname -m')
        .toString()
        .trim();
}
exports.uname = uname;
/**
 * Generates an architecture name that would be used in an Electron or Node.js
 * download file name.
 *
 * Copied from `@electron/get`.
 */
function getNodeArch(arch, configVariables) {
    if (arch === 'arm') {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        switch (configVariables.arm_version) {
            case '6':
                return uname();
            case '7':
            default:
                return 'armv7l';
        }
    }
    return arch;
}
exports.getNodeArch = getNodeArch;
function determineNativePrebuildArch(arch) {
    if (arch === 'armv7l') {
        return 'arm';
    }
    return arch;
}
exports.determineNativePrebuildArch = determineNativePrebuildArch;
/**
 * The extension of `prebuildify`-generated native modules, after the last `.`. This value differs
 * based on whether the target arch is ARM-based.
 */
function determineNativePrebuildExtension(arch) {
    switch (arch) {
        case 'arm64':
            return 'armv8.node';
        case 'armv7l':
            return 'armv7.node';
    }
    return 'node';
}
exports.determineNativePrebuildExtension = determineNativePrebuildExtension;
async function findPrebuildifyModule(modulePath, platform, arch, electronVersion, abi, devDependencies) {
    if (!isPrebuildifyNativeModule(devDependencies)) {
        return false;
    }
    const moduleName = path_1.default.basename(modulePath);
    const nodeArch = getNodeArch(arch, process.config.variables);
    d(`Checking for prebuilds for "${moduleName}"`);
    const prebuildsDir = path_1.default.join(modulePath, 'prebuilds');
    if (!(await fs_extra_1.default.pathExists(prebuildsDir))) {
        d(`Could not find the prebuilds directory at "${prebuildsDir}"`);
        return false;
    }
    const prebuiltModuleDir = path_1.default.join(prebuildsDir, `${platform}-${determineNativePrebuildArch(nodeArch)}`);
    const nativeExt = determineNativePrebuildExtension(nodeArch);
    const electronNapiModuleFilename = path_1.default.join(prebuiltModuleDir, `electron.napi.${nativeExt}`);
    const nodejsNapiModuleFilename = path_1.default.join(prebuiltModuleDir, `node.napi.${nativeExt}`);
    const abiModuleFilename = path_1.default.join(prebuiltModuleDir, `electron.abi${abi}.${nativeExt}`);
    if (await fs_extra_1.default.pathExists(electronNapiModuleFilename) || await fs_extra_1.default.pathExists(nodejsNapiModuleFilename)) {
        node_api_1.ensureElectronSupportsNodeAPI(moduleName, electronVersion);
        d(`Found prebuilt Node-API module in ${prebuiltModuleDir}"`);
    }
    else if (await fs_extra_1.default.pathExists(abiModuleFilename)) {
        d(`Found prebuilt module: "${abiModuleFilename}"`);
    }
    else {
        d(`Could not locate "${electronNapiModuleFilename}", "${nodejsNapiModuleFilename}", or "${abiModuleFilename}"`);
        return false;
    }
    return true;
}
exports.findPrebuildifyModule = findPrebuildifyModule;
//# sourceMappingURL=prebuildify.js.map