"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const path_1 = __importDefault(require("path"));
const prebuildify_1 = require("../src/module-type/prebuildify");
// Copied from @electron/get
describe('uname()', () => {
    if (process.platform !== 'win32') {
        it('should return the correct arch for your system', () => {
            // assumes that the tests will always be run on an x64 system 😬
            chai_1.expect(prebuildify_1.uname()).to.equal('x86_64');
        });
    }
});
// Based on getHostArch tests from @electron/get
describe('getNodeArch()', () => {
    it('should return process.arch on x64', () => {
        chai_1.expect(prebuildify_1.getNodeArch('x64', {})).to.equal('x64');
    });
    it('should return process.arch on ia32', () => {
        chai_1.expect(prebuildify_1.getNodeArch('ia32', {})).to.equal('ia32');
    });
    it('should return process.arch on arm64', () => {
        chai_1.expect(prebuildify_1.getNodeArch('arm64', {})).to.equal('arm64');
    });
    it('should return process.arch on unknown arm', () => {
        chai_1.expect(prebuildify_1.getNodeArch('arm', {})).to.equal('armv7l');
    });
    if (process.platform !== 'win32') {
        it('should return uname on arm 6', () => {
            chai_1.expect(prebuildify_1.getNodeArch('arm', { arm_version: '6' })).to.equal(prebuildify_1.uname());
        });
    }
    it('should return armv7l on arm 7', () => {
        chai_1.expect(prebuildify_1.getNodeArch('arm', { arm_version: '7' })).to.equal('armv7l');
    });
});
describe('determineNativePrebuildArch', () => {
    it('returns arm if passed in armv7l', () => {
        chai_1.expect(prebuildify_1.determineNativePrebuildArch('armv7l')).to.equal('arm');
    });
    it('returns the input arch if the input is not armv7l', () => {
        chai_1.expect(prebuildify_1.determineNativePrebuildArch('x64')).to.equal('x64');
    });
});
describe('determineNativePrebuildExtension', () => {
    it('returns armv8 suffix for an arm64 arch', () => {
        chai_1.expect(prebuildify_1.determineNativePrebuildExtension('arm64')).to.equal('armv8.node');
    });
    it('returns armv7 suffix for an armv7l arch', () => {
        chai_1.expect(prebuildify_1.determineNativePrebuildExtension('armv7l')).to.equal('armv7.node');
    });
    it('returns no suffix for non-ARM arches', () => {
        chai_1.expect(prebuildify_1.determineNativePrebuildExtension('x64')).to.equal('node');
    });
});
describe('isPrebuildifyNativeModule', () => {
    it('succeeds if prebuildify exists in devDependencies', () => {
        chai_1.expect(prebuildify_1.isPrebuildifyNativeModule({ prebuildify: 'validvalue' })).to.equal(true);
    });
    it('fails if prebuildify does not exist in devDependencies', () => {
        chai_1.expect(prebuildify_1.isPrebuildifyNativeModule({})).to.equal(false);
    });
});
describe('findPrebuildifyModule', () => {
    const fixtureBaseDir = path_1.default.join(__dirname, 'fixture', 'prebuildify');
    const platform = 'linux';
    const arch = 'x64';
    const electronVersion = '13.0.0';
    const abi = '89';
    const devDependencies = { prebuildify: '^1.0.0' };
    describe('not a prebuildify native module', () => {
        it('should not find a prebuilt native module', async () => {
            chai_1.expect(await prebuildify_1.findPrebuildifyModule('modulePath', platform, arch, electronVersion, abi, {})).to.equal(false);
        });
    });
    describe('with no prebuilds directory', () => {
        it('should not find a prebuilt native module', async () => {
            const noPrebuildsDir = __dirname;
            chai_1.expect(await prebuildify_1.findPrebuildifyModule(noPrebuildsDir, platform, arch, electronVersion, abi, devDependencies)).to.equal(false);
        });
    });
    describe('with prebuilt module for the given ABI', async () => {
        it('should find a prebuilt native module for x64/electron', async () => {
            const fixtureDir = path_1.default.join(fixtureBaseDir, 'abi');
            chai_1.expect(await prebuildify_1.findPrebuildifyModule(fixtureDir, platform, arch, electronVersion, abi, devDependencies)).to.equal(true);
        });
    });
    describe('with prebuilt Node-API module', async () => {
        it('should find a prebuilt native module for x64/node', async () => {
            const fixtureDir = path_1.default.join(fixtureBaseDir, 'napi');
            chai_1.expect(await prebuildify_1.findPrebuildifyModule(fixtureDir, platform, arch, electronVersion, abi, devDependencies)).to.equal(true);
        });
        it('should find a prebuilt native module for armv7l/node', async () => {
            const fixtureDir = path_1.default.join(fixtureBaseDir, 'napi');
            chai_1.expect(await prebuildify_1.findPrebuildifyModule(fixtureDir, platform, 'armv7l', electronVersion, abi, devDependencies)).to.equal(true);
        });
        it('should find a prebuilt native module for arm64/electron', async () => {
            const fixtureDir = path_1.default.join(fixtureBaseDir, 'napi');
            chai_1.expect(await prebuildify_1.findPrebuildifyModule(fixtureDir, platform, 'arm64', electronVersion, abi, devDependencies)).to.equal(true);
        });
    });
    describe('when it cannot find a prebuilt module', async () => {
        it('should not find a prebuilt native module', async () => {
            const fixtureDir = path_1.default.join(fixtureBaseDir, 'not-found');
            chai_1.expect(await prebuildify_1.findPrebuildifyModule(fixtureDir, platform, arch, electronVersion, abi, devDependencies)).to.equal(false);
        });
    });
});
//# sourceMappingURL=module-type-prebuildify.js.map